@@ -22,7 +22,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "arkode/arkode_arkstep.h"
 #include "arkode_arkstep_impl.h"
+#include "arkode_ls_impl.h"
 #include <sundials/sundials_math.h>
 #include <sundials/sundials_types.h>
 
@@ -202,6 +204,12 @@ char *ARKStepGetReturnFlagName(long int flag) {
   These wrappers for ARKLs module 'get' routines all are
   documented in arkode_arkstep.h.
   ---------------------------------------------------------------*/
+int ARKStepGetJac(void *arkode_mem, SUNMatrix *J) {
+  return arkLSGetJac(arkode_mem, J); }
+int ARKStepGetJacTime(void *arkode_mem, sunrealtype *t_J) {
+  return arkLSGetJacTime(arkode_mem, t_J); }
+int ARKStepGetJacNumSteps(void *arkode_mem, long *nst_J) {
+  return arkLSGetJacNumSteps(arkode_mem, nst_J); }
 int ARKStepGetLinWorkSpace(void *arkode_mem, long int *lenrwLS, long int *leniwLS) {
   return(arkLSGetWorkSpace(arkode_mem, lenrwLS, leniwLS)); }
 int ARKStepGetNumJacEvals(void *arkode_mem, long int *njevals) {
@@ -1078,6 +1086,27 @@ int ARKStepSetTableNum(void *arkode_mem, ARKODE_DIRKTableID itable, ARKODE_ERKTa
 }
 
 
+/*---------------------------------------------------------------
+  ARKStepSetTableName:
+
+  Specifies to use pre-existing Butcher tables for the system,
+  based on the string passed to
+  ARKodeButcherTable_LoadERKByName() and
+  ARKodeButcherTable_LoadDIRKByName() within the files
+  arkode_butcher_erk.c and arkode_butcher_dirk.c (automatically
+  calls ARKStepSetImEx).
+
+  If itable is "ARKODE_DIRK_NONE" or etable is "ARKODE_ERK_NONE",
+  then this disables the corresponding table.
+  ---------------------------------------------------------------*/
+int ARKStepSetTableName(void *arkode_mem, const char *itable, const char *etable)
+{
+  return ARKStepSetTableNum(arkode_mem,
+                            arkButcherTableDIRKNameToID(itable),
+                            arkButcherTableERKNameToID(etable));
+}
+
+
 /*---------------------------------------------------------------
   ARKStepSetNonlinCRDown:
 
